@@ -25,6 +25,8 @@ extern "C" {
 void htmlDocDumpMemory(xmlDocPtr cur, xmlChar**mem, int *size);
 void htmlDocDump(FILE *f, xmlDocPtr cur);
 int htmlSaveFile(const char *filename, xmlDocPtr cur);
+void htmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur);
+void htmlNodeDumpFile(FILE *out, xmlDocPtr doc, xmlNodePtr cur);
 
 #ifdef __cplusplus
 }
