@@ -4,3 +4,5 @@
 #undef HAVE_LIBM
 #undef HAVE_ISINF
 #undef HAVE_ISNAN
+#undef HAVE_LIBHISTORY
+#undef HAVE_LIBREADLINE
